/*jslint nomen: true, indent: 2, maxerr: 3, maxlen: 80*/
/*global window, RSVP, rJS, jIO*/
(function (window, document, RSVP, rJS, jIO) {
 "use strict";
 
 rJS(window)
 
 .ready(function(){
    var index, db;
    
    index = FlexSearch.create("memory");
    
    db = jIO.createJIO(
      {
      type : "indexeddb",
      database : "mynij"
      }
    );
    
    this.changeState({
      index : index,
      db : db,
      msgpack : msgpack
    });
    
    return this._load_index();
 })
 
 .declareMethod("get", function(id){
   return this.state.db.get(id);
 })
 
 .declareMethod("get_loaded", function(){
   var gadget = this;
   return this.state.db.get("loaded")
    .push(function(result){
      return result;
    })
    .push(undefined, function (my_error) {return null});
 })
 
 .declareMethod("add_page", function(page_info){ //page_info = {link, title, description, item}
    console.log(this.state.index.info());
    var gadget = this,
        tmp;
    tmp = page_info;
    tmp.portal_type = "page";
    return gadget.state.db.put(page_info.link, tmp)
      .push(function(){
        return gadget.state.index.add(page_info.link, page_info.title + " " + page_info.item);
      });
 })
 
 .declareMethod("loaded", function(file_name){
    var gadget = this,
      id;
    console.log(file_name + " done loading");
      
    return gadget.state.db.get("loaded")
      .push(function(result){
        var tmp = result;
        tmp[file_name] = true;
        return gadget.state.db.put("loaded", tmp);
      })
      .push(undefined, function (my_error) {
        console.log(my_error);
        var tmp = {};
        tmp[file_name] = true;
        return gadget.state.db.put("loaded", tmp);
      })
      .always(function(){
        return gadget._save_index();
      });
 })
 
 .declareMethod("_save_index", function(){
    var gadget = this;
    console.log("getting index");
    var serialized = this.state.index.export_test();
    console.log("Index received. Saving");
    return this.state.db.put("index", this.state.index.info())
      .push(function(){
        return gadget.state.db.putAttachment("index", "ids", new Blob([serialized.ids], {type : "application/json"}));
      })
      .push(function(){
        return gadget.state.db.putAttachment("index", "map", new Blob([serialized.map], {type : "application/json"}));
      })
      .push(function(){
        return gadget.state.db.putAttachment("index", "ctx", new Blob([serialized.ctx], {type : "application/json"}));
      })
      .push(function(){
        console.log("index saved");
      })
      .push(undefined, function (my_error) {
        console.log(my_error);
      });
 })
 
 .declareMethod("search", function(query){
   return this.state.index.search(query);
 })
 
 .declareMethod("_load_index", function(msgpack){
   var gadget = this,
        ids,
        map;
    return gadget.state.db.getAttachment("index", "ids", {"format": "text"})
      .push(function(result){
        ids = result;
        return gadget.state.db.getAttachment("index", "map", {"format": "text"});
      })
      .push(function(result){
        map = result;
        return gadget.state.db.getAttachment("index", "ctx", {"format": "text"});
      })
      .push(function(result){
        gadget.state.index.import_test(ids, map, result);
        console.log("index imported from memory");
      })
      .push(undefined, function (my_error) {console.log(my_error)});
  })
    
  .declareMethod("is_empty", function(){
    return this.state.db.allDocs()
      .push(function(all_docs){
        return all_docs.data.total_rows === 0;
      });
  })
  
  .declareMethod("get_index", function(){
    console.log(this.state.index.info());
    //return this.state.index.export(this.state.msgpack);
    return this.state.index.export_test();
  })
  
  .declareMethod("add_index", function(ids, map, ctx){
    console.log("adding index");
    //this.state.index.import(serialized_index, this.state.msgpack);
    this.state.index.import_test(ids, map, ctx);
    console.log(this.state.index.info());
  });
 
}(window, document, RSVP, rJS, jIO));